# Helper scrip to download the models from Huggingface
# full model repository is at
# https://huggingface.co/sampsonML/galaxy-score-based-diffusion-models/
from huggingface_hub import hf_hub_download

repo_id = "sampsonML/galaxy-score-based-diffusion-models"

# list of the currently available models
models_avail = ["hsc32", "hsc64", "ztf32", "ztf64", "roman120", "lsst60"]


def show_available_models(*args):
    """
    Print the available models.
    """
    print("Available models:")
    for model in models_avail:
        print(f"- {model}")
    print(
        "see: {https://huggingface.co/sampsonML/galaxy-score-based-diffusion-models/} for more information on the models and their training."
    )


def get_model(model_name, local_dir=None) -> tuple[str, int]:
    if model_name not in models_avail:
        raise ValueError(
            f"Model {model_name} is not available. Available models are: {models_avail}"
        )
    if model_name == "hsc32":
        return hsc32(local_dir)
    elif model_name == "hsc64":
        return hsc64(local_dir)
    elif model_name == "ztf32":
        return ztf32(local_dir)
    elif model_name == "ztf64":
        return ztf64(local_dir)
    elif model_name == "roman120":
        return roman120(local_dir)
    elif model_name == "lsst60":
        return lsst60(local_dir)
    else:
        raise ValueError(
            f"Model {model_name} is not available. Available models are: {models_avail}"
        )


def hsc32(local_dir=None) -> tuple[str, int]:
    """
    Download the HSC 32x32 ScoreNet model from Hugging Face Hub.
    """
    size = 32
    if local_dir is not None:
        path = hf_hub_download(
            repo_id=repo_id,
            filename="eqx_hsc_ScoreNet32.eqx",
            local_dir=local_dir,
        )
    else:
        path = hf_hub_download(repo_id=repo_id, filename="eqx_hsc_ScoreNet32.eqx")
    return path, size


def hsc64(local_dir=None) -> tuple[str, int]:
    """
    Download the HSC 64x64 ScoreNet model from Hugging Face Hub.
    """
    size = 64
    if local_dir is not None:
        path = hf_hub_download(
            repo_id=repo_id,
            filename="eqx_hsc_ScoreNet64.eqx",
            local_dir=local_dir,
        )
    else:
        path = hf_hub_download(repo_id=repo_id, filename="eqx_hsc_ScoreNet64.eqx")
    return path, size


def ztf32(local_dir=None) -> tuple[str, int]:
    """
    Download the ZTF 32x32 ScoreNet model from Hugging Face Hub.
    """
    size = 32
    if local_dir is not None:
        path = hf_hub_download(
            repo_id=repo_id,
            filename="eqx_ZTF_ScoreNet32.eqx",
            local_dir=local_dir,
        )
    else:
        path = hf_hub_download(repo_id=repo_id, filename="eqx_ZTF_ScoreNet32.eqx")
    return path, size


def ztf64(local_dir=None) -> tuple[str, int]:
    """
    Download the ZTF 64x64 ScoreNet model from Hugging Face Hub.
    """
    size = 64
    if local_dir is not None:
        path = hf_hub_download(
            repo_id=repo_id,
            filename="eqx_ZTF_ScoreNet64.eqx",
            local_dir=local_dir,
        )
    else:
        path = hf_hub_download(repo_id=repo_id, filename="eqx_ZTF_ScoreNet64.eqx")
    return path, size


def roman120(local_dir=None) -> tuple[str, int]:
    """
    Download the Roman 120x120 ScoreNet model from Hugging Face Hub.
    """
    size = 120
    if local_dir is not None:
        path = hf_hub_download(
            repo_id=repo_id,
            filename="eqx_roman_ScoreNet120.eqx",
            local_dir=local_dir,
        )
    else:
        path = hf_hub_download(repo_id=repo_id, filename="eqx_roman_Scorenet120.eqx")
    return path, size


def lsst60(local_dir=None) -> tuple[str, int]:
    """
    Download the LSST 60x60 ScoreNet model from Hugging Face Hub.
    """
    size = 60
    if local_dir is not None:
        path = hf_hub_download(
            repo_id=repo_id,
            filename="eqx_lsst_ScoreNet60.eqx",
            local_dir=local_dir,
        )
    else:
        path = hf_hub_download(repo_id=repo_id, filename="eqx_lsst_ScoreNet60.eqx")
    return path, size



import jax.random as jr
import equinox as eqx
from .models_eqx import ScoreNet
from .utils import*
import os
import jax.numpy as jnp

# model params
patch_size = 4
hidden_size = 64
mix_patch_size = 512
mix_hidden_size = 512
num_blocks = 4
t1 = 10.0
int_beta = lambda t: t
key = jr.PRNGKey(42)
model_key, train_key, loader_key, sample_key = jr.split(key, 4)

def print_info(name, size, path):
    if name == 'hsc32' or name == 'hsc64':
        print("HSC prior model")
        print("--------------")
        print(f"This model is trained on Scarlet1 deblends from \nthe HSC survey")
        print(f"Model size: {size}x{size}")
        print(f"Model path: {path}")

    elif name == 'ztf32' or name == 'ztf64':
        print("ZTF prior model")
        print("--------------")
        print(f"This model is trained on simulated ZTF-like galaxies")
        print(f"Model size: {size}x{size}")
        print(f"Model path: {path}")

    elif name == 'roman120':
        print("Roman prior model")
        print("--------------")
        print(f"This model is trained on simulated Roman lensed galaxies")
        print(f"Model size: {size}x{size}")
        print(f"Model path: {path}")

    elif name == 'lsst60':
        print("LSST prior model")
        print("--------------")
        print(f"This model is trained on simulated LSST lensed galaxies")
        print(f"Model size: {size}x{size}")
        print(f"Model path: {path}")

# create prior generation function
def get_prior(name, local_dir=None):
    """ 
    Get the prior model for the given name.
    Parameters
    ----------
    name : str 
    local_dir : str 

    Returns 
    ------- 
    prior_model : ScoreNet 
    """

    path, size = get_model(name, local_dir=local_dir)
    # initialise model
    data_shape = (1, size, size)
    model_ = ScoreNet(
        data_shape,
        patch_size,
        hidden_size,
        mix_patch_size,
        mix_hidden_size,
        num_blocks,
        t1,
        key=model_key,
    )
    prior_ = eqx.tree_deserialise_leaves(path, model_)

    def model_wrapper(x, t=0.01):
        sigma = 0.1
        x_ = jnp.log(x + 1) / sigma
        raw_grad = prior_(x_, t)
        transform_grad = raw_grad * (1 / (sigma * (x + 1)))  # analytic derrivitive
        return transform_grad

    score_func = model_wrapper

    # make a local prior class that contains model information 
    class Prior: 
        def __init__(self, name, size, path):
            self.name = name
            self.size = size
            self.path = path 

        def info(self):
            print_info(self.name, self.size, self.path)
            
        def shape(self):
            return (self.size, self.size)

        def path(self):
            return self.path

        def __call__(self, x, t=0.02):
            return score_func(x, t)

    # instantiate the model class 
    PriorModel = Prior(name, size, path)

    return PriorModel


